@@ -1,5 +1,9 @@
 use strict;
 
+#
+#			Interface Definition Language (OMG IDL CORBA v3.0)
+#
+
 package CcdrVisitor;
 
 # needs $node->{c_name} (CnameVisitor), $node->{c_literal} (CliteralVisitor)
@@ -13,41 +17,79 @@ sub open_stream {
 	$self->{filename} = $filename;
 }
 
+sub _get_defn {
+	my $self = shift;
+	my($defn) = @_;
+	if (ref $defn) {
+		return $defn;
+	} else {
+		return $self->{symbtab}->Lookup($defn);
+	}
+}
+
 #
 #	3.5		OMG IDL Specification		(specialized)
 #
 
 #
-#	3.6		Module Declaration
+#	3.7		Module Declaration
 #
 
+sub visitModules {
+	my $self = shift;
+	my($node) = @_;
+	unless (exists $node->{$self->{num_key}}) {
+		$node->{$self->{num_key}} = 0;
+	}
+	my $module = ${$node->{list_decl}}[$node->{$self->{num_key}}];
+	$module->visit($self);
+	$node->{$self->{num_key}} ++;
+}
+
 sub visitModule {
 	my $self = shift;
 	my($node) = @_;
 	my $FH = $self->{out};
+	my $defn = $self->{symbtab}->Lookup($node->{full});
 	print $FH "/*\n";
-	print $FH " * begin of module ",$node->{c_name},"\n";
+	print $FH " * begin of module ",$defn->{c_name},"\n";
 	print $FH " */\n";
 	foreach (@{$node->{list_decl}}) {
-		$_->visit($self);
+		$self->_get_defn($_)->visit($self);
 	}
 	print $FH "\n";
 	print $FH "/*\n";
-	print $FH " * end of module ",$node->{c_name},"\n";
+	print $FH " * end of module ",$defn->{c_name},"\n";
 	print $FH " */\n";
 	print $FH "\n";
 }
 
 #
-#	3.7		Interface Declaration		(specialized)
+#	3.8		Interface Declaration		(specialized)
 #
 
-sub visitForwardInterface {
+sub visitAbstractInterface {
+	# C mapping is aligned with CORBA 2.1
+}
+
+sub visitLocalInterface {
+	# C mapping is aligned with CORBA 2.1
+}
+
+sub visitForwardRegularInterface {
 	# empty
 }
 
+sub visitForwardAbstractInterface {
+	# C mapping is aligned with CORBA 2.1
+}
+
+sub visitForwardLocalInterface {
+	# C mapping is aligned with CORBA 2.1
+}
+
 #
-#	3.8		Value Declaration
+#	3.9		Value Declaration
 #
 
 sub visitRegularValue {
@@ -71,7 +113,7 @@ sub visitForwardAbstractValue {
 }
 
 #
-#	3.9		Constant Declaration
+#	3.10	Constant Declaration
 #
 
 sub visitConstant {
@@ -79,14 +121,14 @@ sub visitConstant {
 }
 
 #
-#	3.10	Type Declaration
+#	3.11	Type Declaration
 #
 
 sub visitTypeDeclarators {
 	my $self = shift;
 	my($node) = @_;
-	foreach (@{$node->{list_value}}) {
-		$_->visit($self);
+	foreach (@{$node->{list_decl}}) {
+		$self->_get_defn($_)->visit($self);
 	}
 }
 
@@ -94,12 +136,13 @@ sub visitTypeDeclarator {
 	my $self = shift;
 	my($node) = @_;
 	return if (exists $node->{modifier});	# native IDL2.2
-	if (	   $node->{type}->isa('StructType')
-			or $node->{type}->isa('UnionType')
-			or $node->{type}->isa('EnumType')
-			or $node->{type}->isa('SequenceType')
-			or $node->{type}->isa('FixedPtType') ) {
-		$node->{type}->visit($self);
+	my $type = $self->_get_defn($node->{type});
+	if (	   $type->isa('StructType')
+			or $type->isa('UnionType')
+			or $type->isa('EnumType')
+			or $type->isa('SequenceType')
+			or $type->isa('FixedPtType') ) {
+		$type->visit($self);
 	}
 	my $FH = $self->{out};
 	if (exists $node->{array_size}) {
@@ -116,98 +159,98 @@ sub visitTypeDeclarator {
 			$first = 0;
 		}
 		print $FH "#define ADD_SIZE_",$node->{c_name},"(size,v) {\\\n";
-		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+		print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"(v);\\\n";
 		print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"(v) + (",$nb,");\\\n";
 		print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\tADD_SIZE_",$node->{type}->{c_name},"(size,*",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\tADD_SIZE_",$type->{c_name},"(size,*",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t}\\\n";
 		print $FH "\t}\n";
 		print $FH "#define PUT_",$node->{c_name},"(ptr,v) {\\\n";
-		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+		print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"(v);\\\n";
 		print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"(v) + (",$nb,");\\\n";
 		print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\tPUT_",$node->{type}->{c_name},"(ptr,*",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\tPUT_",$type->{c_name},"(ptr,*",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t}\\\n";
 		print $FH "\t}\n";
 		if (defined $node->{length}) {
 			if (exists $self->{client}) {
 				print $FH "#define GET_inout_",$node->{c_name},"(ptr,v) {\\\n";
-				print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-				print $FH "\t\t\tGET_inout_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+				print $FH "\t\t\tGET_inout_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define GET_out_",$node->{c_name},"(ptr,v) {\\\n";
-				print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-				print $FH "\t\t\tGET_out_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+				print $FH "\t\t\tGET_out_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define ALLOC_GET_out_",$node->{c_name},"(ptr,v) {\\\n";
-				print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
-				print $FH "\t\tptr = ",$node->{type}->{c_name},"__alloc(",$nb,");\\\n";
+				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+				print $FH "\t\tptr = ",$type->{c_name},"__alloc(",$nb,");\\\n";
 				print $FH "\t\tif (NULL == ptr) goto err;\\\n";
 				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-				print $FH "\t\t\tGET_out_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+				print $FH "\t\t\tGET_out_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 			} else {
 				print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
-				print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-				print $FH "\t\t\tGET_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+				print $FH "\t\t\tGET_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define FREE_in_",$node->{c_name},"(v) {\\\n";
-				print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-				print $FH "\t\t\tFREE_in_",$node->{type}->{c_name},"(",$node->{c_name},"_ptr);\\\n";
+				print $FH "\t\t\tFREE_in_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define FREE_inout_",$node->{c_name},"(v) {\\\n";
-				print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-				print $FH "\t\t\tFREE_inout_",$node->{type}->{c_name},"(",$node->{c_name},"_ptr);\\\n";
+				print $FH "\t\t\tFREE_inout_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 				print $FH "#define FREE_out_",$node->{c_name},"(v) {\\\n";
-				print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+				print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 				print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 				print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-				print $FH "\t\t\tFREE_out_",$node->{type}->{c_name},"(",$node->{c_name},"_ptr);\\\n";
+				print $FH "\t\t\tFREE_out_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 				print $FH "\t\t}\\\n";
 				print $FH "\t}\n";
 			}
 			print $FH "#define FREE_",$node->{c_name},"(v) {\\\n";
-			print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+			print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 			print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 			print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 			print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-			print $FH "\t\t\tFREE_",$node->{type}->{c_name},"(",$node->{c_name},"_ptr);\\\n";
+			print $FH "\t\t\tFREE_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 			print $FH "\t\t}\\\n";
 			print $FH "\t}\n";
 		} else {
 			print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
-			print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+			print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 			print $FH "\t\tfor (",$node->{c_name},"_ptr = ",$deref,"*(v);\\\n";
 			print $FH "\t\t     ",$node->{c_name},"_ptr < ",$deref,"*(v) + (",$nb,");\\\n";
 			print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-			print $FH "\t\t\tGET_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+			print $FH "\t\t\tGET_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 			print $FH "\t\t}\\\n";
 			print $FH "\t}\n";
 			if (exists $self->{client}) {
@@ -216,20 +259,20 @@ sub visitTypeDeclarator {
 			}
 		}
 	} else {
-		print $FH "#define ADD_SIZE_",$node->{c_name}," ADD_SIZE_",$node->{type}->{c_name},"\n";
-		print $FH "#define PUT_",$node->{c_name}," PUT_",$node->{type}->{c_name},"\n";
-		print $FH "#define GET_",$node->{c_name}," GET_",$node->{type}->{c_name},"\n";
+		print $FH "#define ADD_SIZE_",$node->{c_name}," ADD_SIZE_",$type->{c_name},"\n";
+		print $FH "#define PUT_",$node->{c_name}," PUT_",$type->{c_name},"\n";
+		print $FH "#define GET_",$node->{c_name}," GET_",$type->{c_name},"\n";
 		if (defined $node->{length}) {
 			if (exists $self->{client}) {
-				print $FH "#define GET_inout_",$node->{c_name}," GET_inout_",$node->{type}->{c_name},"\n";
-				print $FH "#define GET_out_",$node->{c_name}," GET_out_",$node->{type}->{c_name},"\n";
-				print $FH "#define ALLOC_GET_out_",$node->{c_name}," ALLOC_GET_out_",$node->{type}->{c_name},"\n";
+				print $FH "#define GET_inout_",$node->{c_name}," GET_inout_",$type->{c_name},"\n";
+				print $FH "#define GET_out_",$node->{c_name}," GET_out_",$type->{c_name},"\n";
+				print $FH "#define ALLOC_GET_out_",$node->{c_name}," ALLOC_GET_out_",$type->{c_name},"\n";
 			} else {
-				print $FH "#define FREE_in_",$node->{c_name}," FREE_in_",$node->{type}->{c_name},"\n";
-				print $FH "#define FREE_inout_",$node->{c_name}," FREE_inout_",$node->{type}->{c_name},"\n";
-				print $FH "#define FREE_out_",$node->{c_name}," FREE_out_",$node->{type}->{c_name},"\n";
+				print $FH "#define FREE_in_",$node->{c_name}," FREE_in_",$type->{c_name},"\n";
+				print $FH "#define FREE_inout_",$node->{c_name}," FREE_inout_",$type->{c_name},"\n";
+				print $FH "#define FREE_out_",$node->{c_name}," FREE_out_",$type->{c_name},"\n";
 			}
-			print $FH "#define FREE_",$node->{c_name}," FREE_",$node->{type}->{c_name},"\n";
+			print $FH "#define FREE_",$node->{c_name}," FREE_",$type->{c_name},"\n";
 		} else {
 			if (exists $self->{client}) {
 				print $FH "#define GET_inout_",$node->{c_name}," GET_",$node->{c_name},"\n";
@@ -241,9 +284,9 @@ sub visitTypeDeclarator {
 }
 
 #
-#	3.10.2	Constructed Types
+#	3.11.2	Constructed Types
 #
-#	3.10.2.1	Structures
+#	3.11.2.1	Structures
 #
 
 sub visitStructType {
@@ -252,11 +295,12 @@ sub visitStructType {
 	return if (exists $self->{done_hash}->{$node->{c_name}});
 	$self->{done_hash}->{$node->{c_name}} = 1;
 	foreach (@{$node->{list_expr}}) {
-		if (	   $_->{type}->isa('StructType')
-				or $_->{type}->isa('UnionType')
-				or $_->{type}->isa('SequenceType')
-				or $_->{type}->isa('FixedPtType') ) {
-			$_->{type}->visit($self);
+		my $type = $self->_get_defn($_->{type});
+		if (	   $type->isa('StructType')
+				or $type->isa('UnionType')
+				or $type->isa('SequenceType')
+				or $type->isa('FixedPtType') ) {
+			$type->visit($self);
 		}
 	}
 	$self->{add_size} = '';
@@ -268,7 +312,7 @@ sub visitStructType {
 	$self->{free} = '';
 	$self->{union} = '';
 	foreach (@{$node->{list_value}}) {
-		$_->visit($self);				# single or array
+		$self->_get_defn($_)->visit($self);		# single or array
 	}
 	my $FH = $self->{out};
 	print $FH "#define ADD_SIZE_",$node->{c_name},"(size,v) {\\\n";
@@ -305,7 +349,6 @@ sub visitStructType {
 		}
 		print $FH "#define FREE_",$node->{c_name},"(v) {\\\n";
 		print $FH $self->{free};
-#		print $FH "\t\tCORBA_free(v);\\\n";
 		print $FH "\t}\n";
 	} else {
 		print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
@@ -341,61 +384,62 @@ sub visitArray {
 		$first = 0;
 	}
 
+	my $type = $self->_get_defn($node->{type});
 	$self->{add_size}  .= "\t\t{\\\n";
-	$self->{add_size}  .= "\t\t\t" . $node->{type}->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
+	$self->{add_size}  .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
 	$self->{add_size}  .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)." . $self->{union} . $node->{c_name} . ");\\\n";
 	$self->{add_size}  .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)." . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
 	$self->{add_size}  .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
-	$self->{add_size}  .= "\t\t\t\tADD_SIZE_" . $node->{type}->{c_name} . "(size,*" . $node->{c_name} . "_ptr);\\\n";
+	$self->{add_size}  .= "\t\t\t\tADD_SIZE_" . $type->{c_name} . "(size,*" . $node->{c_name} . "_ptr);\\\n";
 	$self->{add_size}  .= "\t\t\t}\\\n";
 	$self->{add_size}  .= "\t\t}\\\n";
 	$self->{put}       .= "\t\t{\\\n";
-	$self->{put}       .= "\t\t\t" . $node->{type}->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
+	$self->{put}       .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
 	$self->{put}       .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)." . $self->{union} . $node->{c_name} . ");\\\n";
 	$self->{put}       .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)." . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
 	$self->{put}       .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
-	$self->{put}       .= "\t\t\t\tPUT_" . $node->{type}->{c_name} . "(ptr,*" . $node->{c_name} . "_ptr);\\\n";
+	$self->{put}       .= "\t\t\t\tPUT_" . $type->{c_name} . "(ptr,*" . $node->{c_name} . "_ptr);\\\n";
 	$self->{put}       .= "\t\t\t}\\\n";
 	$self->{put}       .= "\t\t}\\\n";
 	$self->{get}       .= "\t\t{\\\n";
-	$self->{get}       .= "\t\t\t" . $node->{type}->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
+	$self->{get}       .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
 	$self->{get}       .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
 	$self->{get}       .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
 	$self->{get}       .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
-	$self->{get}       .= "\t\t\t\tGET_" . $node->{type}->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
+	$self->{get}       .= "\t\t\t\tGET_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get}       .= "\t\t\t}\\\n";
 	$self->{get}       .= "\t\t}\\\n";
 	$self->{get_in}    .= "\t\t{\\\n";
-	$self->{get_in}    .= "\t\t\t" . $node->{type}->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
+	$self->{get_in}    .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
 	$self->{get_in}    .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
 	$self->{get_in}    .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
 	$self->{get_in}    .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
-	$self->{get_in}    .= "\t\t\t\tGET_in_" . $node->{type}->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
+	$self->{get_in}    .= "\t\t\t\tGET_in_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get_in}    .= "\t\t\t}\\\n";
 	$self->{get_in}    .= "\t\t}\\\n";
 	$self->{get_inout} .= "\t\t{\\\n";
-	$self->{get_inout} .= "\t\t\t" . $node->{type}->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
+	$self->{get_inout} .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
 	$self->{get_inout} .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
 	$self->{get_inout} .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
 	$self->{get_inout} .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
-	$self->{get_inout} .= "\t\t\t\tGET_inout_" . $node->{type}->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
+	$self->{get_inout} .= "\t\t\t\tGET_inout_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get_inout} .= "\t\t\t}\\\n";
 	$self->{get_inout} .= "\t\t}\\\n";
 	$self->{get_out}   .= "\t\t{\\\n";
-	$self->{get_out}   .= "\t\t\t" . $node->{type}->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
+	$self->{get_out}   .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
 	$self->{get_out}   .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
 	$self->{get_out}   .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
 	$self->{get_out}   .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
-	$self->{get_out}   .= "\t\t\t\tGET_" . $node->{type}->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
+	$self->{get_out}   .= "\t\t\t\tGET_" . $type->{c_name} . "(ptr," . $node->{c_name} . "_ptr);\\\n";
 	$self->{get_out}   .= "\t\t\t}\\\n";
 	$self->{get_out}   .= "\t\t}\\\n";
-	if (defined $node->{type}->{length}) {
+	if (defined $type->{length}) {
 		$self->{free}      .= "\t\t{\\\n";
-		$self->{free}      .= "\t\t\t" . $node->{type}->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
+		$self->{free}      .= "\t\t\t" . $type->{c_name} . " * " . $node->{c_name} . "_ptr;\\\n";
 		$self->{free}      .= "\t\t\tfor (" . $node->{c_name} . "_ptr = " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ");\\\n";
 		$self->{free}      .= "\t\t\t     " . $node->{c_name} . "_ptr < " . $deref . "((v)->" . $self->{union} . $node->{c_name} . ") + (" . $nb . ");\\\n";
 		$self->{free}      .= "\t\t\t     " . $node->{c_name} . "_ptr++) {\\\n";
-		$self->{free}      .= "\t\t\t\tFREE_" . $node->{type}->{c_name} . "(" . $node->{c_name} . "_ptr);\\\n";
+		$self->{free}      .= "\t\t\t\tFREE_" . $type->{c_name} . "(" . $node->{c_name} . "_ptr);\\\n";
 		$self->{free}      .= "\t\t\t}\\\n";
 		$self->{free}      .= "\t\t}\\\n";
 	}
@@ -406,25 +450,26 @@ sub visitSingle {
 	my($node) = @_;
 	my $tab = '';
 	$tab = "\t" if ($self->{union});
-	$self->{add_size}  .= $tab . "\t\tADD_SIZE_" . $node->{type}->{c_name};
+	my $type = $self->_get_defn($node->{type});
+	$self->{add_size}  .= $tab . "\t\tADD_SIZE_" . $type->{c_name};
 		$self->{add_size}  .= "(size,(v)." . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{put}       .= $tab . "\t\tPUT_" . $node->{type}->{c_name};
+	$self->{put}       .= $tab . "\t\tPUT_" . $type->{c_name};
 		$self->{put}       .= "(ptr,(v)." . $self->{union} . $node->{c_name} . ");\\\n";
-	$self->{get}       .= $tab . "\t\tGET_" . $node->{type}->{c_name};
+	$self->{get}       .= $tab . "\t\tGET_" . $type->{c_name};
 		$self->{get}       .= "(ptr,&((v)->" . $self->{union} . $node->{c_name} . "));\\\n";
-	$self->{get_in}    .= $tab . "\t\tGET_in_" . $node->{type}->{c_name};
+	$self->{get_in}    .= $tab . "\t\tGET_in_" . $type->{c_name};
 		$self->{get_in}    .= "(ptr,&((v)->" . $self->{union} . $node->{c_name} . "));\\\n";
-	$self->{get_inout} .= $tab . "\t\tGET_inout_" . $node->{type}->{c_name};
+	$self->{get_inout} .= $tab . "\t\tGET_inout_" . $type->{c_name};
 		$self->{get_inout} .= "(ptr,&((v)->" . $self->{union} . $node->{c_name} . "));\\\n";
-	$self->{get_out}   .= $tab . "\t\tGET_out_" . $node->{type}->{c_name};
+	$self->{get_out}   .= $tab . "\t\tGET_out_" . $type->{c_name};
 		$self->{get_out}   .= "(ptr,&((v)->" . $self->{union} . $node->{c_name} . "));\\\n";
-	if (defined $node->{type}->{length}) {
-		$self->{free}  .= $tab . "\t\tFREE_" . $node->{type}->{c_name};
+	if (defined $type->{length}) {
+		$self->{free}  .= $tab . "\t\tFREE_" . $type->{c_name};
 			$self->{free}   .= "(&((v)->" . $self->{union} . $node->{c_name} . "));\\\n";
 	}
 }
 
-#	3.10.2.2	Discriminated Unions
+#	3.11.2.2	Discriminated Unions
 #
 
 sub visitUnionType {
@@ -433,12 +478,12 @@ sub visitUnionType {
 	return if (exists $self->{done_hash}->{$node->{c_name}});
 	$self->{done_hash}->{$node->{c_name}} = 1;
 	foreach (@{$node->{list_expr}}) {
-		if (	   $_->{element}->{type}->isa('StructType')
-				or $_->{element}->{type}->isa('UnionType')
-				or $_->{element}->{type}->isa('EnumType')
-				or $_->{element}->{type}->isa('SequenceType')
-				or $_->{element}->{type}->isa('FixedPtType') ) {
-			$_->{element}->{type}->visit($self);
+		my $type = $self->_get_defn($_->{element}->{type});
+		if (	   $type->isa('StructType')
+				or $type->isa('UnionType')
+				or $type->isa('SequenceType')
+				or $type->isa('FixedPtType') ) {
+			$type->visit($self);
 		}
 	}
 	my $FH = $self->{out};
@@ -453,14 +498,15 @@ sub visitUnionType {
 	foreach (@{$node->{list_expr}}) {
 		$_->visit($self);				# case
 	}
+	my $type = $self->_get_defn($node->{type});
 	print $FH "#define ADD_SIZE_",$node->{c_name},"(size,v) {\\\n";
-	print $FH "\t\tADD_SIZE_",$node->{type}->{c_name},"(size,(v)._d);\\\n";
+	print $FH "\t\tADD_SIZE_",$type->{c_name},"(size,(v)._d);\\\n";
 	print $FH "\t\tswitch ((v)._d) {\\\n";
 	print $FH $self->{add_size};
 	print $FH "\t\t}\\\n";
 	print $FH "\t}\n";
 	print $FH "#define PUT_",$node->{c_name},"(ptr,v) {\\\n";
-	print $FH "\t\tPUT_",$node->{type}->{c_name},"(ptr,(v)._d);\\\n";
+	print $FH "\t\tPUT_",$type->{c_name},"(ptr,(v)._d);\\\n";
 	print $FH "\t\tswitch ((v)._d) {\\\n";
 	print $FH $self->{put};
 	print $FH "\t\t}\\\n";
@@ -468,13 +514,13 @@ sub visitUnionType {
 	if (defined $node->{length}) {
 		if (exists $self->{client}) {
 			print $FH "#define GET_inout_",$node->{c_name},"(ptr,v) {\\\n";
-			print $FH "\t\tGET_inout_",$node->{type}->{c_name},"(ptr,&((v)->_d));\\\n";
+			print $FH "\t\tGET_inout_",$type->{c_name},"(ptr,&((v)->_d));\\\n";
 			print $FH "\t\tswitch ((v)->_d) {\\\n";
 			print $FH $self->{get_inout};
 			print $FH "\t\t}\\\n";
 			print $FH "\t}\n";
 			print $FH "#define GET_out_",$node->{c_name},"(ptr,v) {\\\n";
-			print $FH "\t\tGET_out_",$node->{type}->{c_name},"(ptr,&((v)->_d));\\\n";
+			print $FH "\t\tGET_out_",$type->{c_name},"(ptr,&((v)->_d));\\\n";
 			print $FH "\t\tswitch ((v)->_d) {\\\n";
 			print $FH $self->{get_out};
 			print $FH "\t\t}\\\n";
@@ -482,14 +528,14 @@ sub visitUnionType {
 			print $FH "#define ALLOC_GET_out_",$node->{c_name},"(ptr,v) {\\\n";
 			print $FH "\t\tv = ",$node->{c_name},"__alloc(1);\\\n";
 			print $FH "\t\tif (NULL == (v)) goto err;\\\n";
-			print $FH "\t\tGET_out_",$node->{type}->{c_name},"(ptr,&((v)->_d));\\\n";
+			print $FH "\t\tGET_out_",$type->{c_name},"(ptr,&((v)->_d));\\\n";
 			print $FH "\t\tswitch ((v)->_d) {\\\n";
 			print $FH $self->{get_out};
 			print $FH "\t\t}\\\n";
 			print $FH "\t}\n";
 		} else {
 			print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
-			print $FH "\t\tGET_",$node->{type}->{c_name},"(ptr,&((v)->_d));\\\n";
+			print $FH "\t\tGET_",$type->{c_name},"(ptr,&((v)->_d));\\\n";
 			print $FH "\t\tswitch ((v)->_d) {\\\n";
 			print $FH $self->{get};
 			print $FH "\t\t}\\\n";
@@ -506,11 +552,10 @@ sub visitUnionType {
 		print $FH "\t\tswitch ((v)->_d) {\\\n";
 		print $FH $self->{free};
 		print $FH "\t\t}\\\n";
-#		print $FH "\t\tCORBA_free(v);\\\n";
 		print $FH "\t}\n";
 	} else {
 		print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
-		print $FH "\t\tGET_",$node->{type}->{c_name},"(ptr,&((v)->_d));\\\n";
+		print $FH "\t\tGET_",$type->{c_name},"(ptr,&((v)->_d));\\\n";
 		print $FH "\t\tswitch ((v)->_d) {\\\n";
 		print $FH $self->{get};
 		print $FH "\t\t}\\\n";
@@ -554,7 +599,7 @@ sub visitCase {
 			$self->{free}      .= "\t\tcase " . $_->{c_literal} . ":\\\n";
 		}
 	}
-	$node->{element}->{value}->visit($self);	# array or single
+	$self->_get_defn($node->{element}->{value})->visit($self);		# single or array
 	$self->{add_size}  .= "\t\tbreak;\\\n";
 	$self->{put}       .= "\t\tbreak;\\\n";
 	$self->{get}       .= "\t\tbreak;\\\n";
@@ -564,16 +609,7 @@ sub visitCase {
 	$self->{free}      .= "\t\tbreak;\\\n";
 }
 
-
-#	3.10.2.3	Enumerations
-#
-
-sub visitEnumType {
-	# empty
-}
-
-#
-#	3.10.3	Constructed Recursive Types and Forward Declarations
+#	3.11.2.3	Constructed Recursive Types and Forward Declarations
 #
 
 sub visitForwardStructType {
@@ -584,43 +620,51 @@ sub visitForwardUnionType {
 	# empty
 }
 
+#	3.11.2.4	Enumerations
 #
-#	3.10.4	Template Types
+
+sub visitEnumType {
+	# empty
+}
+
+#
+#	3.11.3	Template Types
 #
 
 sub visitSequenceType {
 	my $self = shift;
 	my($node) = @_;
-	if (	   $node->{type}->isa('SequenceType')
-			or $node->{type}->isa('FixedPtType') ) {
-		$node->{type}->visit($self);
+	my $type = $self->_get_defn($node->{type});
+	if (	   $type->isa('SequenceType')
+			or $type->isa('FixedPtType') ) {
+		$type->visit($self);
 	}
 	my $FH = $self->{out};
 	print $FH "#ifndef _ALIGN_",$node->{c_name},"_defined\n";
 	print $FH "#define _ALIGN_",$node->{c_name},"_defined\n";
 	print $FH "#define ADD_SIZE_",$node->{c_name},"(size,v) {\\\n";
-	print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+	print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 	print $FH "\t\tADD_SIZE_CORBA_unsigned_long(size,(v)._length);\\\n";
 	print $FH "\t\tfor (",$node->{c_name},"_ptr = (v)._buffer;\\\n";
 	print $FH "\t\t     ",$node->{c_name},"_ptr < (v)._buffer + (v)._length;\\\n";
 	print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-	print $FH "\t\t\tADD_SIZE_",$node->{type}->{c_name},"(size,*",$node->{c_name},"_ptr);\\\n";
+	print $FH "\t\t\tADD_SIZE_",$type->{c_name},"(size,*",$node->{c_name},"_ptr);\\\n";
 	print $FH "\t\t}\\\n";
 	print $FH "\t}\n";
 	print $FH "#define PUT_",$node->{c_name},"(ptr,v) {\\\n";
-	print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+	print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 	print $FH "\t\tPUT_CORBA_unsigned_long(ptr,(v)._length);\\\n";
 	print $FH "\t\tfor (",$node->{c_name},"_ptr = (v)._buffer;\\\n";
 	print $FH "\t\t     ",$node->{c_name},"_ptr < (v)._buffer + (v)._length;\\\n";
 	print $FH "\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-	print $FH "\t\t\tPUT_",$node->{type}->{c_name},"(ptr,*",$node->{c_name},"_ptr);\\\n";
+	print $FH "\t\t\tPUT_",$type->{c_name},"(ptr,*",$node->{c_name},"_ptr);\\\n";
 	print $FH "\t\t}\\\n";
 	print $FH "\t}\n";
 	my $nb = "(v)->_length";
 	if (exists $self->{client}) {
 		$nb = $node->{max}->{c_literal} if (exists $node->{max});
 		print $FH "#define GET_inout_",$node->{c_name},"(ptr,v) {\\\n";
-		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+		print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tGET_CORBA_unsigned_long(ptr,&((v)->_length));\\\n";
 		print $FH "\t\tif (NULL != (v)->_buffer) CORBA_free((v)->_buffer);\\\n";
 		print $FH "\t\tif (0 != ",$nb,") {\\\n";
@@ -629,14 +673,14 @@ sub visitSequenceType {
 		print $FH "\t\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\t\tGET_inout_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\t\tGET_inout_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t\t}\\\n";
 		print $FH "\t\t} else {\\\n";
 		print $FH "\t\t\t(v)->_buffer = NULL;\\\n";
 		print $FH "\t\t}\\\n";
 		print $FH "\t}\n";
 		print $FH "#define GET_out_",$node->{c_name},"(ptr,v) {\\\n";
-		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+		print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tGET_CORBA_unsigned_long(ptr,&((v)->_length));\\\n";
 		print $FH "\t\tif (0 != ",$nb,") {\\\n";
 		print $FH "\t\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
@@ -644,7 +688,7 @@ sub visitSequenceType {
 		print $FH "\t\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\t\tGET_out_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\t\tGET_out_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t\t}\\\n";
 		print $FH "\t\t} else {\\\n";
 		print $FH "\t\t\t(v)->_buffer = NULL;\\\n";
@@ -653,7 +697,7 @@ sub visitSequenceType {
 		print $FH "#define ALLOC_GET_out_",$node->{c_name}," GET_out_",$node->{c_name},"\n";
 	} else {
 		print $FH "#define GET_",$node->{c_name},"(ptr,v) {\\\n";
-		print $FH "\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+		print $FH "\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\tGET_CORBA_unsigned_long(ptr,&((v)->_length));\\\n";
 		print $FH "\t\tif (0 != (v)->_length) {\\\n";
 		print $FH "\t\t\t(v)->_buffer = ",$node->{c_name},"__allocbuf(",$nb,");\\\n";
@@ -661,7 +705,7 @@ sub visitSequenceType {
 		print $FH "\t\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-		print $FH "\t\t\t\tGET_",$node->{type}->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\t\tGET_",$type->{c_name},"(ptr,",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t\t}\\\n";
 		print $FH "\t\t} else {\\\n";
 		print $FH "\t\t\t(v)->_buffer = NULL;\\\n";
@@ -678,13 +722,12 @@ sub visitSequenceType {
 	}
 	print $FH "#define FREE_",$node->{c_name},"(v) {\\\n";
 	print $FH "\t\tif (NULL != (v)->_buffer) {\\\n";
-	if (defined $node->{type}->{length}) {
-		print $FH "\t\t\t",$node->{type}->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
+	if (defined $type->{length}) {
+		print $FH "\t\t\t",$type->{c_name}," * ",$node->{c_name},"_ptr;\\\n";
 		print $FH "\t\t\tfor (",$node->{c_name},"_ptr = (v)->_buffer;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr < (v)->_buffer + (v)->_length;\\\n";
 		print $FH "\t\t\t     ",$node->{c_name},"_ptr++) {\\\n";
-#		print $FH "\t\t\t\tFREE_",$node->{type}->{c_name},"(*",$node->{c_name},"_ptr);\\\n";
-		print $FH "\t\t\t\tFREE_",$node->{type}->{c_name},"(",$node->{c_name},"_ptr);\\\n";
+		print $FH "\t\t\t\tFREE_",$type->{c_name},"(",$node->{c_name},"_ptr);\\\n";
 		print $FH "\t\t\t}\\\n";
 	}
 	print $FH "\t\t\tCORBA_free((v)->_buffer);\\\n";
@@ -700,8 +743,14 @@ sub visitFixedPtType {
 	warn __PACKAGE__,"::visitFixedPtType : TODO.\n";
 }
 
+sub visitFixedPtConstType {
+	my $self = shift;
+	my($node) = @_;
+	warn __PACKAGE__,"::visitFixedPtConstType : TODO.\n";
+}
+
 #
-#	3.11	Exception Declaration
+#	3.12	Exception Declaration
 #
 
 sub visitException {
@@ -709,11 +758,12 @@ sub visitException {
 	my($node) = @_;
 	return unless (exists $node->{list_expr});
 	foreach (@{$node->{list_expr}}) {
-		if (	   $_->{type}->isa('StructType')
-				or $_->{type}->isa('UnionType')
-				or $_->{type}->isa('SequenceType')
-				or $_->{type}->isa('FixedPtType') ) {
-			$_->{type}->visit($self);
+		my $type = $self->_get_defn($_->{type});
+		if (	   $type->isa('StructType')
+				or $type->isa('UnionType')
+				or $type->isa('SequenceType')
+				or $type->isa('FixedPtType') ) {
+			$type->visit($self);
 		}
 	}
 	$self->{add_size} = '';
@@ -725,7 +775,7 @@ sub visitException {
 	$self->{free} = '';
 	$self->{union} = '';
 	foreach (@{$node->{list_value}}) {
-		$_->visit($self);			# single or array
+		$self->_get_defn($_)->visit($self);		# single or array
 	}
 	my $FH = $self->{out};
 	if (exists $self->{client}) {
@@ -762,11 +812,11 @@ sub visitException {
 }
 
 #
-#	3.12	Operation Declaration		(specialized)
+#	3.13	Operation Declaration		(specialized)
 #
 
 #
-#	3.13	Attribute Declaration
+#	3.14	Attribute Declaration
 #
 
 sub visitAttribute {
@@ -776,5 +826,57 @@ sub visitAttribute {
 	$node->{_set}->visit($self) if (exists $node->{_set});
 }
 
+#
+#	3.15	Repository Identity Related Declarations
+#
+
+sub visitTypeId {
+	# empty
+}
+
+sub visitTypePrefix {
+	# empty
+}
+
+#
+#	3.16	Event Declaration
+#
+
+sub visitRegularEvent {
+	# C mapping is aligned with CORBA 2.1
+}
+
+sub visitAbstractEvent {
+	# C mapping is aligned with CORBA 2.1
+}
+
+sub visitForwardRegularEvent {
+	# C mapping is aligned with CORBA 2.1
+}
+
+sub visitForwardAbstractEvent {
+	# C mapping is aligned with CORBA 2.1
+}
+
+#
+#	3.17	Component Declaration
+#
+
+sub visitComponent {
+	# C mapping is aligned with CORBA 2.1
+}
+
+sub visitForwardComponent {
+	# C mapping is aligned with CORBA 2.1
+}
+
+#
+#	3.18	Home Declaration
+#
+
+sub visitHome {
+	# C mapping is aligned with CORBA 2.1
+}
+
 1;
 
